-- vertex

#version 330 core

uniform mat4 proj;
uniform mat4 mv;
uniform mat3 nm;
uniform	vec3 eye;	// eye position
uniform vec3 light;     // light position

layout ( location = 0 ) in vec3 position;
layout ( location = 1 ) in vec2 texCoord;
layout ( location = 2 ) in vec3 normal;
layout ( location = 3 ) in vec3 tangent;
layout ( location = 4 ) in vec3 binormal;

out vec3  n;
out vec3  v;
out vec3  l;
out vec3  h;
out vec2  tex;
out vec3  nw;
out vec3  pw;
out vec3  pl;
out vec3  nl;
void main(void)
{
	vec4 	p = mv * vec4 ( position, 1.0 );
	vec3	n = normalize ( nm * normal );
	vec3	t = normalize ( nm * tangent );
	vec3	b = normalize ( nm * binormal );
	
	gl_Position  = proj * p;
	n            = normalize ( nm * normal );
	nw           = vec3 ( mv * vec4 ( normal, 1.0 ) );
	pw           = p.xyz;
	pl           = position.xyz;
	nl           = normal;
	v            = normalize ( eye - p.xyz );					// vector to the eye
	l            = normalize ( light - p.xyz );
	h            = normalize ( l + v );	
	v            = vec3 ( dot ( v, t ), dot ( v, b ), dot ( v, n ) );
	l            = vec3 ( dot ( l, t ), dot ( l, b ), dot ( l, n ) );
	h            = vec3 ( dot ( h, t ), dot ( h, b ), dot ( h, n ) );
	tex          = texCoord / 2;
}

-- fragment
#version 330 core

in  vec3 n;
in  vec3 v;
in  vec3 l;
in  vec3 h;
in  vec2 tex;
in  vec3 nw;		// world-space normal
in  vec3 pw;		// world-space position
in  vec3 pl;
in  vec3 nl;
out vec4 color;

uniform sampler2D rainMap;
uniform sampler2D rainDripsMap;
uniform sampler2D rainDripsMaskMap;
uniform sampler2D bumpMap;
uniform sampler2D colorMap;
uniform float	  time;

//const vec4	diffColor = vec4 ( 0.5, 0.0, 0.0, 1.0 );
const vec4	specColor = vec4 ( 1.0 );
const float	specPower = 70.0;

//const float rainSpeed       = 1.0;
//const float edgeWidth2      = 0.05;
const float PI              = 3.1415926;
//const float waterBrightness = 0.4;
//const vec4  waterColor      = vec4 ( 0.5, 0.7, 1.0, 1.0 );
const float ka   = 0.2;
const float kd   = 0.8;
const float ks   = 0.3;
const float waterRoughness = 0.07;
const float waterSpecular  = 0.3;
const vec3  lum            = vec3 ( 0.2126, 0.7152, 0.0722 );
const float texScale       = 0.25;
const float rainSpeed      = 0.7;

float	sqr ( float x )
{
	return x;
}

    // optimized x^5 without special functions
float pow5 ( in float x )
{
    float x2 = x * x;
    
    return x2 * x2 * x;
}

float	saturate (  float x )
{
	return clamp ( x, 0, 1 );
}

float	frac ( float x )
{
	return mod ( x, 1.0 );
}

vec3	saturate ( in vec3 x )
{
	return clamp ( x, vec3(0.0), vec3(1.0) );
}

vec3	desaturate ( in vec3 color, float percent )
{
	vec3	d  = vec3 ( dot ( color,lum  ) );		// completely desaturated  color

	return d * percent + color * (1.0 - percent );
}


void	water ( inout vec3 color, inout float roughness, inout float specular, in float porousness, in float wetness  )
{
	vec3	des      = desaturate ( color, -0.5 );
	vec3	wetColor = clamp ( des, vec3(0.0), vec3(1.0) ) * 0.5;

	color     = mix ( color, wetColor, porousness * wetness );
	roughness = mix ( roughness, waterRoughness, wetness );
	specular  = mix ( specular,  waterSpecular,  wetness );
}

void main(void)
{
	vec3	n    = 2.0 * (texture ( bumpMap, tex ).rgb - vec3 ( 0.5 ));
	vec3	nn   = normalize ( n );
	vec3	v2   = normalize ( v );
	vec3	l2   = normalize ( l );
	vec3	h2   = normalize ( h );
	vec4    clr  = texture ( colorMap, tex );

	vec3    p1    = pw * vec3 ( 1,  -1,  0.5 );
	vec3    p2    = pw * vec3 ( -1, -1, -0.5 );
	vec3    mask1 = saturate ( sign ( nw ) );
	float   mask2 = round    ( abs ( nw ) ).r;
	vec3    q1    = mix ( p2, p1, mask1.g );
	vec3    q2    = mix ( p1, p2, mask1.r );
	vec2    tx    = mix ( q1.xy, q2.zy, mask2 );

vec2 na = abs ( nl.xz );
na /= na.x + na.y;

tx = vec2 ( pl.x * na.x + pl.z * na.y, pl.y );
tx = tex;

vec4	m0 = texture ( rainDripsMap, tx * texScale );	// RG - n.xy, B-mask, A-temp offs

vec3	temp  = vec3(round (m0.b), 0.2, 0.75 );
vec3	temp2 = vec3(round (m0.b), 0.03, 0.0125 );
vec3	tampOut = mix( temp, temp2, 1.0 );	// control speed over material

float	t    = mix (temp.g, temp.b, m0.a) * ( time + m0.a );
vec4	m1   = texture ( rainDripsMaskMap, vec2(t) + 0.5*tx );
float	mask = m1.r * round(m0.b);

vec3	n2 = normalize( vec3((m0.rg * 2.0 - vec2(1.0))*mask,1.0));


	vec4 diff = clr * max ( dot ( nn, l2 ), 0.1 );
	vec4 spec = 2*specColor * pow ( max ( dot ( n2, h2 ), 0.0 ), mix (50,100, mask) );

	color = diff + ks * vec4 ( spec * mask);

//color = vec4(mask);
color = vec4(spec);
//color=vec4(m0.b);
//color=texture(rainDripsMaskMap, tx);
//color = texture(rainDripsMap,tx/4);
}
