#include	"GlutRotateWindow.h"
#include	"Program.h"
#include	"BasicMesh.h"
#include	"Texture.h"

class	MeshWindow : public GlutRotateWindow
{
	Program		program;
	Texture		rainDropsMap;
	Texture		rainNormalMap;
	Texture		bumpMap;
	Texture		colorMap;
	BasicMesh * mesh  = nullptr;
	glm::vec3   light = glm::vec3  ( 9, 9, 9 );
	const float	size = 2;
	
public:
	MeshWindow () : GlutRotateWindow ( 100, 50, 900, 900, "Bumpmapping" )
	{
		if ( !program.loadProgram ( "rain.glsl" ) )
			exit ( "Error building program: %s\n", program.getLog ().c_str () );
		
		if ( !colorMap.load2D ( "../../Textures/Fieldstone.dds" ) )
			exit ( "Error loading decal map\n" );

		if ( !bumpMap.load2D  ( "../../Textures/Bumpmaps/FieldStone-DOT3.tga" ) )
			exit ( "Error loaing bumpmap\n" );
			
		if ( !rainDropsMap.load2D ( "rain_drops.tga" ) )
			exit ( "Error loading rain drops map\n" );
		
		program.bind       ();
		program.setTexture ( "rainDropsMap",  0 );
		program.setTexture ( "rainNormalMap", 1 );
		program.setTexture ( "bumpMap",       2 );
		program.setTexture ( "colorMap",      3 );
		
		mesh  = createBox  ( glm::vec3 ( -size ), glm::vec3 ( 2.0f * size ) );
		eye   = glm::vec3  ( -8, 0, 0 );
	}

	void redisplay () override
	{
		const glm::mat4	mv = getRotation  ();
		
		glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

		rainDropsMap.bind        ( 0 );
		colorMap.bind            ( 3 );
		program.setUniformMatrix ( "mv", mv );
		program.setUniformMatrix ( "nm", normalMatrix ( mv ) );

		mesh -> render ();
	}

	void reshape ( int w, int h ) override
	{
		GlutWindow::reshape ( w, h );
		
		program.setUniformMatrix ( "proj", getProjection () );
	}

	void mouseWheel ( int wheel, int dir, int x, int y ) override
	{
		eye += glm::vec3 ( dir, 0, 0 ) * 0.5f;
		
					// since eye value has changed
		reshape ( getWidth(), getHeight() );
		
		glutPostRedisplay ();
	}
	
	void	idle () override
	{
		const float	angle = 4 * getTime ();
		
		light.x = 9*cos ( angle );
		light.y = 9*sin ( 1.4 * angle );
		light.z = 9 * sin ( angle / 3 );

		program.setUniformVector ( "light", eye ); //light );
		program.setUniformVector ( "eye",   eye );
		program.setUniformFloat  ( "time",  getTime () );
		
		glutPostRedisplay ();
	}
};

int main ( int argc, char * argv [] )
{
	GlutWindow::init ( argc, argv );
	
	MeshWindow	win;
	
	return GlutWindow::run ();
}
